VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CResource"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Description = "Handles resource scripts (.rc) and compiles them to compiled resource scripts (.res)."
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

'****************************************************
'* CResource.cls                                    *
'* Resource create & handle class                   *
'* Programmed: Konrad Rudolph, Mad Rat Pro.         *
'* Last Change: 24/11/2001 22:00                    *
'* Freeware: do with it whatever you want to as     *
'*  long as it's no commercial use!                 *
'* Version: 2.0.2                                   *
'****************************************************
'* Allows to create and handle resource files with  *
'* the following features:                          *
'*  - bitmaps (no compressing, .bmp)                *
'*  - waves (sound buffer, .wav)                    *
'*  - tables (text & data access)                   *
'*  - strings (data storage in one string)          *
'*  - if nothing fits, just store a binary file     *
'*  - all this stuff in one file                    *
'*  - access using keywords (IDs)                   *
'*  - creating of resource script files             *
'*  - bases on scripts (*.rc), compiles to *.res    *
'*    files (compiled resource files)               *
'****************************************************

Private Const Copyright = "2001-2002 by Mad Rat Pro."

'include <defResource.bas>

Private iNumber As Integer                  ' number of resources
Private iCount As New Collection            ' number to enumerate resources
Private sSources As New Collection          ' paths of the source files
Private sID As New Collection               ' IDs of resources
Private bType As New Collection             ' types of resources
Private lSize As New Collection             ' sizes of resources
Private lStart As New Collection            ' first byte of resource
Private InfoHeader() As rcInfo              ' resource information for header

' returns the number of resources
Public Property Get Count() As Integer
    Count = iNumber
End Property

'// as all the IDs in the following properties are set to Variant you may use
'// both, the key as well as the number of the resource to access the data!

' sets or returns the path of a resource
Public Property Get Path(ByVal ID As Variant) As String
    Path = sSources.Item(ID)
End Property

Public Property Let Path(ByVal ID As Variant, ByVal sNewValue As String)
    Dim Path As String, ff As Integer
    
    Path = sNewValue
    If Dir$(Path) = "" Then Exit Property
    sSources.Remove ID
    sSources.Add Path, sID.Item(ID)
    ff = FreeFile
    Open Path For Binary As #ff
    lSize.Remove ID
    lSize.Add LOF(ff), sID.Item(ID)
    Close #ff
End Property

' returns the ID of a resource
Public Property Get ID(ByVal Index As Integer) As String
    ID = sID.Item(Index)
End Property

' returns the number of a resource
Public Property Get Index(ByVal ID As Variant) As Integer
    Index = iCount.Item(ID)
End Property

' sets or returns the type of a resource
Public Property Get rcType(ByVal ID As Variant) As Byte
    rcType = bType.Item(ID)
End Property

Public Property Let rcType(ByVal ID As Variant, ByVal bNewValue As Byte)
    bType.Remove ID
    bType.Add bNewValue, sID.Item(ID)
End Property

' returns then size
Public Property Get Size(ByVal ID As Variant) As Long
    Size = lSize.Item(ID)
End Property

' adds a new resource, returns ID when successful
Public Function Add(Path As String, ID As String, rcType As rcTypes) As String
    Dim ff As Integer
    
    Add = ""
    If Dir$(Path) = "" Then Exit Function
    On Error GoTo ErrorAdd
    If sID.Item(ID) <> "" Then
        On Error GoTo 0
        Exit Function
    End If
ErrorAdd:
    On Error GoTo 0
    
    ff = FreeFile
    Open Path For Binary As #ff
    sSources.Add Path, ID
    sID.Add ID, ID
    bType.Add CByte(rcType), ID
    lSize.Add LOF(ff), ID
    iNumber = iNumber + 1
    iCount.Add iNumber, ID
    Close #ff
    Add = ID
End Function

' delectes a resource
Public Function Remove(ID As Variant) As Boolean
    Dim Dummy As Variant, I As Integer
        
    Remove = False
    If iNumber = 0 Then Exit Function
    On Error GoTo ErrorRemove
    Dummy = sID.Item(ID)
    On Error GoTo 0
    
    sSources.Remove ID
    sID.Remove ID
    bType.Remove ID
    lSize.Remove ID
    iCount.Remove ID
    For I = 1 To iCount.Count
        If iCount.Item(I) = I + 1 Then
            iCount.Remove I
            iCount.Add I, sID.Item(I)
        End If
    Next
    iNumber = iNumber - 1
    Remove = True
    
ErrorRemove:
    On Error GoTo 0
End Function

' writes rc script: ToC
Public Sub WriteToC(ByVal Path As String)
    Dim ff As Integer, I As Integer
    
    If iNumber = 0 Then Exit Sub
    If LCase$(Right$(Path, 3)) <> ".rc" Then Path = Path + ".rc"
    If Dir$(Path) <> "" Then Kill Path
    ff = FreeFile
    Open Path For Output As #ff
    Print #ff, "Count: " & iNumber
    For I = 1 To iNumber
        Print #ff, sID.Item(I) & " ";
        Print #ff, sSources.Item(I) & " " & bType.Item(I)
    Next
    Close #ff
End Sub

' loads rc script: ToC
Public Sub ReadToC(ByVal Path As String)
    Dim ff As Integer, f2 As Integer, I As Integer
    Dim A As String, Space As Integer
    
    If LCase$(Right$(Path, 3)) <> ".rc" Then Path = Path + ".rc"
    If Dir$(Path) = "" Then Exit Sub
    ff = FreeFile
    On Error GoTo LoadToCError
    Open Path For Input As #ff
    Input #ff, A
    If Not InStr(LCase$(A), "count: ") = 1 Then Exit Sub
    iNumber = CInt(Right$(A, Len(A) - InStr(A, " ")))
    Set sSources = New Collection
    Set sID = New Collection
    Set bType = New Collection
    Set iCount = New Collection
    Set lSize = New Collection
    Set lStart = New Collection
    For I = 1 To iNumber
        Input #ff, A
        Space = InStr(A, " ")
        sID.Add Left$(A, Space - 1), Left$(A, Space - 1)
        A = Right$(A, Len(A) - Space)
        Space = InStr(A, " ")
        sSources.Add Left$(A, Space - 1), sID.Item(I)
        A = Right$(A, Len(A) - Space)
        bType.Add CByte(A), sID.Item(I)
        f2 = FreeFile
        Open sSources.Item(I) For Binary As #f2
        If LOF(f2) = 0 Then
            Close #f2
            Kill sSources.Item(I)
            sSources.Remove sID.Item(I)
            bType.Remove sID.Item(I)
            sID.Remove sID.Item(I)
            I = I - 1
        Else
            lSize.Add LOF(f2), sID.Item(I)
            iCount.Add CInt(I), sID.Item(I)
            Close #f2
        End If
    Next
    
LoadToCError:
    On Error GoTo 0
End Sub

' compiles the resources to one compiled resource script
' returns the size of the destination file
Public Function Compile(ByVal Path As String) As Long
    Dim ff As Integer, f2 As Integer, I As Integer
    Dim A As String
    Dim Header As rcHeader
    ReDim InfoHeader(1 To iNumber) As rcInfo
    
    Compile = 0
    On Error GoTo ErrorCompile
    If iNumber = 0 Then Exit Function
    If Path = "" Then Exit Function
    If LCase$(Right$(Path, 4)) <> ".res" Then Path = Path + ".res"
    If Dir$(Path) <> "" Then Kill Path
    'fill header & infoheaders...
    Header.iNumber = iNumber
    Header.lSize = 6 + iNumber * 24           ' size of the file
    For I = 1 To iNumber
        With InfoHeader(I)
            .bType = bType.Item(I)
            .lBegin = Header.lSize + 1
            .lSize = lSize.Item(I)
            .sKeyID = sID.Item(I)
            Header.lSize = Header.lSize + .lSize
        End With
    Next
    
    ff = FreeFile
    Open Path For Binary As #ff
    '...write headers into file...
    Put #ff, , Header
    Put #ff, , InfoHeader()
    '...and fill it out with the resources!
    For I = 1 To iNumber
        f2 = FreeFile
        Open sSources.Item(I) For Binary As #f2
        A = Space$(LOF(f2))
        Get #f2, , A
        Put #ff, , A
        Close #f2
    Next
    Compile = LOF(ff)
    Close #ff
    On Error GoTo 0
    Exit Function
    
ErrorCompile:
    Compile = 0
    On Error GoTo 0
End Function

